## Benchmarking
Currently shotover-proxy carries a 27% overhead on pure throughput in bypass mode (no query enrichment).
Not surprising as we are still copying a lot of bytes around and its quite hard to just pass a shared buffer around.
shotover-proxy is a message based proxy, so we still process the C* frame and do stuff with it (e.g. create native rust types
from the underlying byte stream).

The plan to improve (reduce) the amount of copying that occurs, is to introduce a "branded" lifetime that spans the entire
request and response lifespan. The constructed underlying message will just be pointers into the buffer rather than copied values.
Only when we need to consume, modify or explicitly copy the bytes will we then do so. 

Running shotover-proxy - 4 threads, cassandra source and cassandra-destination codec - debug build
- `./tools/bin/cassandra-stress write no-warmup -mode native cql3 connectionsPerHost=8 -rate threads=8 -node 127.0.0.1 -port native=9043`

```
Connected to cluster: Test Cluster, max pending requests per connection 128, max connections per host 8
Datatacenter: datacenter1; Host: /127.0.0.1; Rack: rack1
Created keyspaces. Sleeping 1s for propagation.
Sleeping 2s...
Running WRITE with 8 threads until stderr of mean < 0.02
type       total ops,    op/s,    pk/s,   row/s,    mean,     med,     .95,     .99,    .999,     max,   time,   stderr, errors,  gc: #,  max ms,  sum ms,  sdv ms,      mb
total,           727,     727,     727,     727,     4.8,     4.3,     7.8,    25.1,    34.5,    35.7,    1.0,  0.00000,      0,      0,       0,       0,       0,       0
total,          2811,    2084,    2084,    2084,     3.8,     3.0,     8.2,    18.6,    28.6,    30.1,    2.0,  0.33978,      0,      0,       0,       0,       0,       0
total,          5272,    2461,    2461,    2461,     3.2,     2.7,     6.7,    11.3,    16.9,    17.6,    3.0,  0.24204,      0,      0,       0,       0,       0,       0
total,          8075,    2803,    2803,    2803,     2.8,     2.3,     6.4,    12.5,    18.9,    20.0,    4.0,  0.19322,      0,      0,       0,       0,       0,       0
total,         11808,    3733,    3733,    3733,     2.1,     1.9,     3.6,     4.8,     8.2,     9.0,    5.0,  0.18326,      0,      0,       0,       0,       0,       0
total,         15393,    3585,    3585,    3585,     2.2,     1.9,     4.0,     5.7,    35.0,    43.0,    6.0,  0.15978,      0,      0,       0,       0,       0,       0
total,         21184,    5791,    5791,    5791,     1.4,     1.3,     2.0,     2.9,     3.9,     5.1,    7.0,  0.17979,      0,      0,       0,       0,       0,       0
total,         27002,    5818,    5818,    5818,     1.4,     1.3,     2.0,     2.6,     4.2,     5.1,    8.0,  0.16994,      0,      0,       0,       0,       0,       0
total,         31687,    4685,    4685,    4685,     1.7,     1.5,     2.8,     5.5,    20.3,    30.9,    9.0,  0.15019,      0,      0,       0,       0,       0,       0
total,         35863,    4176,    4176,    4176,     1.9,     1.8,     3.3,     4.3,     5.9,     7.7,   10.0,  0.13385,      0,      0,       0,       0,       0,       0
total,         39651,    3788,    3788,    3788,     2.1,     1.9,     3.7,     5.8,     9.3,    10.6,   11.0,  0.12121,      0,      0,       0,       0,       0,       0
total,         43582,    3931,    3931,    3931,     2.0,     1.8,     3.6,     5.3,     8.0,    10.6,   12.0,  0.11054,      0,      0,       0,       0,       0,       0
total,         48573,    4991,    4991,    4991,     1.6,     1.4,     2.6,     4.1,    34.9,    38.5,   13.0,  0.10314,      0,      0,       0,       0,       0,       0
total,         53887,    5314,    5314,    5314,     1.5,     1.3,     2.3,     3.4,    43.1,    44.0,   14.0,  0.09767,      0,      1,      41,      41,       0,     624
total,         57885,    3998,    3998,    3998,     2.0,     1.8,     3.6,     5.5,    11.6,    13.1,   15.0,  0.09099,      0,      0,       0,       0,       0,       0
total,         62843,    4958,    4958,    4958,     1.6,     1.5,     2.6,     3.9,    12.9,    14.5,   16.0,  0.08546,      0,      0,       0,       0,       0,       0
total,         68760,    5917,    5917,    5917,     1.3,     1.3,     1.9,     2.6,     3.6,     4.4,   17.0,  0.08267,      0,      0,       0,       0,       0,       0
total,         73816,    5056,    5056,    5056,     1.6,     1.4,     2.6,     4.5,     7.3,     9.1,   18.0,  0.07810,      0,      0,       0,       0,       0,       0
total,         77668,    3852,    3852,    3852,     2.1,     1.9,     3.5,     4.8,     7.0,     7.9,   19.0,  0.07435,      0,      0,       0,       0,       0,       0
total,         81265,    3597,    3597,    3597,     2.2,     2.0,     3.9,     5.4,     6.9,     7.7,   20.0,  0.07137,      0,      0,       0,       0,       0,       0
total,         85215,    3950,    3950,    3950,     2.0,     1.7,     3.9,     5.9,    18.1,    22.8,   21.0,  0.06805,      0,      0,       0,       0,       0,       0
total,         90771,    5556,    5556,    5556,     1.4,     1.3,     2.2,     3.0,     4.4,     6.0,   22.0,  0.06577,      0,      0,       0,       0,       0,       0
total,         94353,    3582,    3582,    3582,     2.2,     2.0,     3.7,     5.8,    11.9,    14.0,   23.0,  0.06354,      0,      0,       0,       0,       0,       0
total,         98076,    3723,    3723,    3723,     2.1,     1.9,     3.8,     6.7,    13.8,    16.0,   24.0,  0.06125,      0,      0,       0,       0,       0,       0
total,        101838,    3762,    3762,    3762,     2.1,     2.0,     3.6,     4.8,     9.1,    11.4,   25.0,  0.05909,      0,      0,       0,       0,       0,       0
total,        106704,    4866,    4866,    4866,     1.6,     1.4,     3.0,     4.4,     6.3,     8.3,   26.0,  0.05684,      0,      0,       0,       0,       0,       0
total,        111609,    4905,    4905,    4905,     1.6,     1.4,     2.4,     4.8,    56.1,    58.0,   27.0,  0.05497,      0,      1,      54,      54,       0,     593
total,        116285,    4676,    4676,    4676,     1.7,     1.6,     2.6,     3.3,     4.5,     5.2,   28.0,  0.05294,      0,      0,       0,       0,       0,       0
total,        122115,    5830,    5830,    5830,     1.4,     1.3,     2.0,     2.5,     3.5,     4.9,   29.0,  0.05207,      0,      0,       0,       0,       0,       0
total,        127743,    5628,    5628,    5628,     1.4,     1.3,     2.2,     3.7,     5.7,     7.5,   30.0,  0.05089,      0,      0,       0,       0,       0,       0
total,        133000,    5257,    5257,    5257,     1.5,     1.4,     2.4,     3.4,     6.8,     8.2,   31.0,  0.04940,      0,      0,       0,       0,       0,       0
total,        139061,    6061,    6061,    6061,     1.3,     1.3,     1.8,     2.2,     2.7,     3.0,   32.0,  0.04877,      0,      0,       0,       0,       0,       0
total,        144575,    5514,    5514,    5514,     1.4,     1.3,     2.2,     3.2,     5.5,     7.0,   33.0,  0.04754,      0,      0,       0,       0,       0,       0
total,        150350,    5775,    5775,    5775,     1.4,     1.3,     2.0,     2.4,     3.4,     5.0,   34.0,  0.04657,      0,      0,       0,       0,       0,       0
total,        156165,    5815,    5815,    5815,     1.4,     1.3,     1.9,     3.3,     6.0,     7.8,   35.0,  0.04565,      0,      0,       0,       0,       0,       0
total,        161842,    5677,    5677,    5677,     1.4,     1.3,     2.1,     2.9,     9.4,    10.3,   36.0,  0.04465,      0,      0,       0,       0,       0,       0
total,        167333,    5491,    5491,    5491,     1.4,     1.3,     1.9,     2.6,    70.1,    71.4,   37.0,  0.04380,      0,      1,      68,      68,       0,     589
total,        173158,    5825,    5825,    5825,     1.4,     1.3,     2.0,     2.6,     4.0,     4.7,   38.0,  0.04293,      0,      0,       0,       0,       0,       0
total,        178914,    5756,    5756,    5756,     1.4,     1.3,     1.9,     2.9,    13.7,    23.3,   39.0,  0.04210,      0,      0,       0,       0,       0,       0
total,        184849,    5935,    5935,    5935,     1.3,     1.3,     1.8,     2.3,     2.9,     3.6,   40.0,  0.04132,      0,      0,       0,       0,       0,       0
total,        190033,    5184,    5184,    5184,     1.5,     1.4,     2.4,     4.3,     7.3,     7.9,   41.0,  0.04030,      0,      0,       0,       0,       0,       0
total,        195994,    5961,    5961,    5961,     1.3,     1.3,     1.9,     2.4,     3.6,     4.3,   42.0,  0.03960,      0,      0,       0,       0,       0,       0
total,        201283,    5289,    5289,    5289,     1.5,     1.4,     2.3,     2.8,     3.5,     3.8,   43.0,  0.03867,      0,      0,       0,       0,       0,       0
total,        206013,    4730,    4730,    4730,     1.7,     1.6,     2.7,     3.4,     4.3,     4.7,   44.0,  0.03779,      0,      0,       0,       0,       0,       0
total,        211533,    5520,    5520,    5520,     1.4,     1.3,     2.1,     3.0,     7.1,     9.2,   45.0,  0.03700,      0,      0,       0,       0,       0,       0
total,        216934,    5401,    5401,    5401,     1.5,     1.4,     2.2,     3.5,     6.6,     7.2,   46.0,  0.03621,      0,      0,       0,       0,       0,       0
total,        222845,    5911,    5911,    5911,     1.3,     1.3,     1.9,     2.3,     3.8,     4.8,   47.0,  0.03562,      0,      0,       0,       0,       0,       0
total,        228133,    5288,    5288,    5288,     1.5,     1.3,     2.0,     2.7,    84.0,    86.0,   48.0,  0.03498,      0,      1,      82,      82,       0,     579
total,        234021,    5888,    5888,    5888,     1.3,     1.3,     1.9,     2.3,     3.5,     4.4,   49.0,  0.03441,      0,      0,       0,       0,       0,       0
total,        239616,    5595,    5595,    5595,     1.4,     1.3,     2.0,     3.2,     6.0,     8.4,   50.0,  0.03376,      0,      0,       0,       0,       0,       0
total,        244882,    5266,    5266,    5266,     1.5,     1.4,     2.4,     3.2,     4.6,     6.0,   51.0,  0.03309,      0,      0,       0,       0,       0,       0
total,        250281,    5399,    5399,    5399,     1.5,     1.4,     2.2,     3.8,    12.1,    13.6,   52.0,  0.03246,      0,      0,       0,       0,       0,       0
total,        255993,    5712,    5712,    5712,     1.4,     1.3,     2.0,     2.4,     3.0,     3.7,   53.0,  0.03191,      0,      0,       0,       0,       0,       0
total,        261513,    5520,    5520,    5520,     1.4,     1.4,     2.2,     3.1,     4.1,     5.1,   54.0,  0.03133,      0,      0,       0,       0,       0,       0
total,        266803,    5290,    5290,    5290,     1.5,     1.4,     2.3,     4.2,     7.1,     8.5,   55.0,  0.03075,      0,      0,       0,       0,       0,       0
total,        272173,    5370,    5370,    5370,     1.5,     1.4,     2.3,     3.1,     4.1,     4.9,   56.0,  0.03020,      0,      0,       0,       0,       0,       0
total,        276471,    4298,    4298,    4298,     1.8,     1.7,     3.1,     4.3,    11.5,    13.4,   57.0,  0.02980,      0,      0,       0,       0,       0,       0
total,        280950,    4479,    4479,    4479,     1.8,     1.6,     2.8,     4.3,    10.2,    12.4,   58.0,  0.02936,      0,      0,       0,       0,       0,       0
total,        285452,    4502,    4502,    4502,     1.8,     1.5,     2.6,     4.3,    86.5,    87.9,   59.0,  0.02886,      0,      1,      84,      84,       0,     603
total,        290018,    4566,    4566,    4566,     1.7,     1.6,     2.9,     3.9,     4.9,     6.5,   60.0,  0.02843,      0,      0,       0,       0,       0,       0
total,        294262,    4244,    4244,    4244,     1.9,     1.7,     3.2,     4.6,     6.2,     8.3,   61.0,  0.02810,      0,      0,       0,       0,       0,       0
total,        299422,    5160,    5160,    5160,     1.5,     1.4,     2.6,     4.1,     7.2,     8.2,   62.0,  0.02763,      0,      0,       0,       0,       0,       0
total,        304818,    5396,    5396,    5396,     1.5,     1.4,     2.3,     3.6,     5.2,     5.4,   63.0,  0.02720,      0,      0,       0,       0,       0,       0
total,        310847,    6029,    6029,    6029,     1.3,     1.3,     1.8,     2.2,     2.6,     2.8,   64.0,  0.02692,      0,      0,       0,       0,       0,       0
total,        316615,    5768,    5768,    5768,     1.4,     1.3,     2.0,     2.5,     4.0,     5.5,   65.0,  0.02657,      0,      0,       0,       0,       0,       0
total,        322044,    5429,    5429,    5429,     1.5,     1.4,     2.2,     2.9,     4.2,     5.1,   66.0,  0.02617,      0,      0,       0,       0,       0,       0
total,        327672,    5628,    5628,    5628,     1.4,     1.4,     2.0,     2.7,     4.0,     4.9,   67.0,  0.02581,      0,      0,       0,       0,       0,       0
total,        333094,    5422,    5422,    5422,     1.5,     1.4,     2.2,     3.4,     6.9,     9.9,   68.0,  0.02544,      0,      0,       0,       0,       0,       0
total,        338643,    5549,    5549,    5549,     1.4,     1.3,     2.2,     3.2,     5.3,     7.3,   69.0,  0.02509,      0,      0,       0,       0,       0,       0
total,        343755,    5112,    5112,    5112,     1.6,     1.3,     2.2,     4.2,    81.2,    82.8,   70.0,  0.02474,      0,      1,      79,      79,       0,     576
total,        349444,    5689,    5689,    5689,     1.4,     1.3,     2.1,     2.6,     3.3,     3.7,   71.0,  0.02443,      0,      0,       0,       0,       0,       0
total,        354961,    5517,    5517,    5517,     1.4,     1.4,     2.1,     2.7,     4.0,     4.8,   72.0,  0.02410,      0,      0,       0,       0,       0,       0
total,        358256,    3295,    3295,    3295,     2.4,     2.1,     4.1,     7.3,    31.7,    34.8,   73.0,  0.02427,      0,      0,       0,       0,       0,       0
total,        362493,    4237,    4237,    4237,     1.9,     1.7,     2.9,     4.6,    12.2,    35.4,   74.0,  0.02403,      0,      0,       0,       0,       0,       0
total,        367454,    4961,    4961,    4961,     1.6,     1.5,     2.4,     3.2,     4.2,     7.4,   75.0,  0.02371,      0,      0,       0,       0,       0,       0
total,        372813,    5359,    5359,    5359,     1.5,     1.4,     2.3,     3.3,     4.9,     5.8,   76.0,  0.02340,      0,      0,       0,       0,       0,       0
total,        378428,    5615,    5615,    5615,     1.4,     1.3,     2.1,     4.0,     5.6,     6.4,   77.0,  0.02312,      0,      0,       0,       0,       0,       0
total,        383337,    4909,    4909,    4909,     1.6,     1.4,     2.3,     3.1,    80.4,    81.1,   78.0,  0.02282,      0,      1,      78,      78,       0,     607
total,        389196,    5859,    5859,    5859,     1.4,     1.3,     1.9,     2.4,     3.6,     4.3,   79.0,  0.02259,      0,      0,       0,       0,       0,       0
total,        394834,    5638,    5638,    5638,     1.4,     1.3,     2.0,     4.0,     6.4,     9.3,   80.0,  0.02233,      0,      0,       0,       0,       0,       0
total,        400382,    5548,    5548,    5548,     1.4,     1.3,     2.2,     3.0,     4.3,     5.9,   81.0,  0.02207,      0,      0,       0,       0,       0,       0
total,        406338,    5956,    5956,    5956,     1.3,     1.3,     1.8,     2.3,     3.5,     3.8,   82.0,  0.02187,      0,      0,       0,       0,       0,       0
total,        411808,    5470,    5470,    5470,     1.4,     1.4,     2.2,     2.9,     5.7,     7.3,   83.0,  0.02161,      0,      0,       0,       0,       0,       0
total,        417782,    5974,    5974,    5974,     1.3,     1.3,     1.8,     2.2,     3.0,     4.1,   84.0,  0.02143,      0,      0,       0,       0,       0,       0
total,        423520,    5738,    5738,    5738,     1.4,     1.3,     1.9,     3.6,    10.9,    14.3,   85.0,  0.02121,      0,      0,       0,       0,       0,       0
total,        429095,    5575,    5575,    5575,     1.4,     1.4,     2.0,     2.7,     3.7,     5.1,   86.0,  0.02098,      0,      0,       0,       0,       0,       0
total,        434682,    5587,    5587,    5587,     1.4,     1.3,     2.1,     3.5,     6.7,     9.4,   87.0,  0.02075,      0,      0,       0,       0,       0,       0
total,        439163,    4481,    4481,    4481,     1.8,     1.5,     2.8,     4.1,    65.1,    65.5,   88.0,  0.02054,      0,      1,      64,      64,       0,     607
total,        444994,    5831,    5831,    5831,     1.4,     1.3,     1.9,     2.4,     3.2,     4.7,   89.0,  0.02034,      0,      0,       0,       0,       0,       0
total,        450840,    5846,    5846,    5846,     1.4,     1.3,     2.0,     2.7,     4.3,     5.3,   90.0,  0.02016,      0,      0,       0,       0,       0,       0
total,        455884,    5044,    5044,    5044,     1.6,     1.4,     2.7,     4.3,    11.4,    16.0,   91.0,  0.01993,      0,      0,       0,       0,       0,       0
total,        455900,    4486,    4486,    4486,     1.5,     1.4,     2.2,     2.6,     2.6,     2.6,   91.0,  0.03073,      0,      0,       0,       0,       0,       0


Results:
Op rate                   :    5,010 op/s  [WRITE: 5,010 op/s]
Partition rate            :    5,010 pk/s  [WRITE: 5,010 pk/s]
Row rate                  :    5,010 row/s [WRITE: 5,010 row/s]
Latency mean              :    1.6 ms [WRITE: 1.6 ms]
Latency median            :    1.4 ms [WRITE: 1.4 ms]
Latency 95th percentile   :    2.7 ms [WRITE: 2.7 ms]
Latency 99th percentile   :    4.2 ms [WRITE: 4.2 ms]
Latency 99.9th percentile :    9.8 ms [WRITE: 9.8 ms]
Latency max               :   87.9 ms [WRITE: 87.9 ms]
Total partitions          :    455,900 [WRITE: 455,900]
Total errors              :          0 [WRITE: 0]
Total GC count            : 8
Total GC memory           : 4.666 GiB
Total GC time             :    0.6 seconds
Avg GC time               :   68.8 ms
StdDev GC time            :   14.2 ms
Total operation time      : 00:01:31
```

Running direct - `./tools/bin/cassandra-stress write no-warmup -mode native cql3 connectionsPerHost=8 -rate threads=8 -node 127.0.0.1 -port native=9043`

```
Running WRITE with 8 threads until stderr of mean < 0.02
type       total ops,    op/s,    pk/s,   row/s,    mean,     med,     .95,     .99,    .999,     max,   time,   stderr, errors,  gc: #,  max ms,  sum ms,  sdv ms,      mb
total,          3373,    3373,    3373,    3373,     1.6,     1.2,     2.9,     8.6,    31.0,   143.8,    1.0,  0.00000,      0,      0,       0,       0,       0,       0
total,         10199,    6826,    6826,    6826,     1.1,     0.7,     2.2,    10.4,    27.7,    64.8,    2.0,  0.21138,      0,      0,       0,       0,       0,       0
total,         20292,   10093,   10093,   10093,     0.8,     0.5,     1.6,     4.9,    23.7,    79.1,    3.0,  0.22370,      0,      0,       0,       0,       0,       0
total,         33112,   12820,   12820,   12820,     0.6,     0.4,     1.3,     4.0,    20.0,    57.5,    4.0,  0.20433,      0,      0,       0,       0,       0,       0
total,         43798,   10686,   10686,   10686,     0.7,     0.4,     1.5,     5.0,    25.6,   105.8,    5.0,  0.16318,      0,      1,      98,      98,       0,     569
total,         55676,   11878,   11878,   11878,     0.7,     0.4,     1.4,     5.4,    33.3,    86.0,    6.0,  0.13832,      0,      0,       0,       0,       0,       0
total,         68660,   12984,   12984,   12984,     0.6,     0.4,     1.3,     4.0,    22.0,   107.5,    7.0,  0.12415,      0,      0,       0,       0,       0,       0
total,         82654,   13994,   13994,   13994,     0.6,     0.3,     1.3,     3.7,    22.9,   114.8,    8.0,  0.11497,      0,      0,       0,       0,       0,       0
total,         96995,   14341,   14341,   14341,     0.5,     0.3,     1.2,     3.5,    15.3,   125.2,    9.0,  0.10687,      0,      0,       0,       0,       0,       0
total,        108924,   11929,   11929,   11929,     0.7,     0.4,     1.2,     4.4,    26.9,   230.4,   10.0,  0.09735,      0,      1,     103,     103,       0,     591
total,        123716,   14792,   14792,   14792,     0.5,     0.3,     1.2,     3.9,    16.2,    69.0,   11.0,  0.08966,      0,      0,       0,       0,       0,       0
total,        137411,   13695,   13695,   13695,     0.6,     0.3,     1.3,     4.7,    30.0,    60.2,   12.0,  0.08205,      0,      0,       0,       0,       0,       0
total,        153246,   15835,   15835,   15835,     0.5,     0.3,     1.2,     3.1,    14.1,    89.7,   13.0,  0.07812,      0,      0,       0,       0,       0,       0
total,        170740,   17494,   17494,   17494,     0.4,     0.3,     0.9,     2.9,    10.6,    44.8,   14.0,  0.07531,      0,      0,       0,       0,       0,       0
total,        186160,   15420,   15420,   15420,     0.5,     0.3,     1.0,     3.0,    11.0,   100.0,   15.0,  0.07111,      0,      1,      95,      95,       0,     577
total,        203655,   17495,   17495,   17495,     0.4,     0.3,     1.0,     2.8,     8.1,    23.0,   16.0,  0.06785,      0,      0,       0,       0,       0,       0
total,        219742,   16087,   16087,   16087,     0.5,     0.3,     1.1,     2.9,    15.0,   103.1,   17.0,  0.06476,      0,      0,       0,       0,       0,       0
total,        236075,   16333,   16333,   16333,     0.4,     0.3,     1.2,     2.6,     7.2,    42.8,   18.0,  0.06141,      0,      0,       0,       0,       0,       0
total,        252758,   16683,   16683,   16683,     0.5,     0.3,     1.0,     2.8,     8.5,   108.2,   19.0,  0.05923,      0,      1,     105,     105,       0,     593
total,        269003,   16245,   16245,   16245,     0.5,     0.3,     1.2,     3.0,    11.0,    41.2,   20.0,  0.05635,      0,      0,       0,       0,       0,       0
total,        286659,   17656,   17656,   17656,     0.4,     0.3,     1.1,     2.8,     7.9,    18.8,   21.0,  0.05407,      0,      0,       0,       0,       0,       0
total,        302150,   15491,   15491,   15491,     0.5,     0.3,     1.1,     2.9,    12.1,   101.0,   22.0,  0.05171,      0,      1,      97,      97,       0,     582
total,        318530,   16380,   16380,   16380,     0.5,     0.3,     1.3,     3.2,    13.4,    56.2,   23.0,  0.04957,      0,      0,       0,       0,       0,       0
total,        335406,   16876,   16876,   16876,     0.5,     0.3,     1.1,     3.3,    13.6,    36.9,   24.0,  0.04763,      0,      0,       0,       0,       0,       0
total,        349624,   14218,   14218,   14218,     0.5,     0.3,     1.4,     4.8,    18.3,    54.8,   25.0,  0.04575,      0,      0,       0,       0,       0,       0
total,        364511,   14887,   14887,   14887,     0.5,     0.3,     1.0,     3.2,    16.0,   114.3,   26.0,  0.04399,      0,      1,     104,     104,       0,     581
total,        380387,   15876,   15876,   15876,     0.5,     0.3,     1.2,     3.7,    17.0,    90.6,   27.0,  0.04246,      0,      0,       0,       0,       0,       0
total,        394244,   13857,   13857,   13857,     0.6,     0.3,     1.4,     4.2,    19.1,    68.7,   28.0,  0.04100,      0,      0,       0,       0,       0,       0
total,        407793,   13549,   13549,   13549,     0.6,     0.3,     1.1,     3.8,    35.3,   181.9,   29.0,  0.03957,      0,      1,      89,      89,       0,     614
total,        423849,   16056,   16056,   16056,     0.5,     0.3,     1.2,     3.1,    15.1,    37.4,   30.0,  0.03824,      0,      0,       0,       0,       0,       0
total,        441073,   17224,   17224,   17224,     0.5,     0.3,     1.1,     3.0,     8.1,    54.8,   31.0,  0.03724,      0,      0,       0,       0,       0,       0
total,        458753,   17680,   17680,   17680,     0.4,     0.3,     1.0,     3.0,    10.4,    21.5,   32.0,  0.03625,      0,      0,       0,       0,       0,       0
total,        475535,   16782,   16782,   16782,     0.5,     0.3,     1.0,     2.6,     9.7,    97.2,   33.0,  0.03541,      0,      1,      75,      75,       0,     570
total,        493475,   17940,   17940,   17940,     0.4,     0.3,     1.0,     2.5,     7.0,    16.8,   34.0,  0.03453,      0,      0,       0,       0,       0,       0
total,        510709,   17234,   17234,   17234,     0.5,     0.3,     1.1,     2.7,     7.3,    21.3,   35.0,  0.03359,      0,      0,       0,       0,       0,       0
total,        527684,   16975,   16975,   16975,     0.5,     0.3,     1.1,     2.9,    10.0,    53.5,   36.0,  0.03274,      0,      0,       0,       0,       0,       0
total,        542946,   15262,   15262,   15262,     0.5,     0.3,     1.0,     2.9,    15.6,   111.8,   37.0,  0.03187,      0,      1,     102,     102,       0,     583
total,        560727,   17781,   17781,   17781,     0.4,     0.3,     0.9,     2.8,     7.0,    26.9,   38.0,  0.03115,      0,      0,       0,       0,       0,       0
total,        577977,   17250,   17250,   17250,     0.5,     0.3,     1.1,     2.7,     9.3,    51.1,   39.0,  0.03044,      0,      0,       0,       0,       0,       0
total,        594216,   16239,   16239,   16239,     0.5,     0.3,     1.0,     3.4,    11.7,    30.6,   40.0,  0.02967,      0,      0,       0,       0,       0,       0
total,        609628,   15412,   15412,   15412,     0.5,     0.3,     0.9,     2.8,     9.6,   126.2,   41.0,  0.02897,      0,      1,     122,     122,       0,     581
total,        624866,   15238,   15238,   15238,     0.5,     0.3,     1.1,     3.6,    18.8,    75.6,   42.0,  0.02827,      0,      0,       0,       0,       0,       0
total,        642044,   17178,   17178,   17178,     0.5,     0.3,     1.1,     2.6,     8.8,    48.4,   43.0,  0.02768,      0,      0,       0,       0,       0,       0
total,        658983,   16939,   16939,   16939,     0.4,     0.3,     1.0,     3.1,     8.1,    18.2,   44.0,  0.02705,      0,      0,       0,       0,       0,       0
total,        674561,   15578,   15578,   15578,     0.5,     0.3,     1.0,     3.0,    15.4,   101.1,   45.0,  0.02646,      0,      1,      88,      88,       0,     585
total,        692378,   17817,   17817,   17817,     0.4,     0.3,     1.0,     2.4,     5.6,    19.4,   46.0,  0.02595,      0,      0,       0,       0,       0,       0
total,        709792,   17414,   17414,   17414,     0.4,     0.3,     1.1,     2.4,     7.5,    23.5,   47.0,  0.02543,      0,      0,       0,       0,       0,       0
total,        721915,   12123,   12123,   12123,     0.6,     0.3,     1.3,     4.7,    41.0,   116.4,   48.0,  0.02519,      0,      1,     254,     254,       0,     582
total,        733839,   11924,   11924,   11924,     0.7,     0.3,     1.1,     4.3,    35.1,   319.0,   49.0,  0.02469,      0,      0,       0,       0,       0,       0
total,        747838,   13999,   13999,   13999,     0.6,     0.3,     1.3,     4.4,    33.2,   112.7,   50.0,  0.02421,      0,      0,       0,       0,       0,       0
total,        762995,   15157,   15157,   15157,     0.5,     0.3,     1.1,     4.6,    22.3,    86.4,   51.0,  0.02373,      0,      0,       0,       0,       0,       0
total,        774024,   11029,   11029,   11029,     0.7,     0.3,     1.4,     6.7,    52.1,   226.6,   52.0,  0.02344,      0,      1,     143,     143,       0,     600
total,        789030,   15006,   15006,   15006,     0.5,     0.3,     0.9,     3.2,    37.1,   107.9,   53.0,  0.02300,      0,      0,       0,       0,       0,       0
total,        804904,   15874,   15874,   15874,     0.5,     0.3,     1.1,     3.4,    14.0,    86.7,   54.0,  0.02258,      0,      0,       0,       0,       0,       0
total,        820329,   15425,   15425,   15425,     0.5,     0.3,     1.3,     3.8,    14.4,    66.2,   55.0,  0.02216,      0,      0,       0,       0,       0,       0
total,        835244,   14915,   14915,   14915,     0.5,     0.3,     1.1,     3.5,    15.6,   120.7,   56.0,  0.02176,      0,      1,      76,      76,       0,     594
total,        852044,   16800,   16800,   16800,     0.5,     0.3,     1.1,     3.0,     9.1,    24.5,   57.0,  0.02139,      0,      0,       0,       0,       0,       0
total,        868716,   16672,   16672,   16672,     0.5,     0.3,     1.0,     3.1,    12.3,    49.1,   58.0,  0.02103,      0,      0,       0,       0,       0,       0
total,        885387,   16671,   16671,   16671,     0.5,     0.3,     1.2,     3.0,     8.4,    26.6,   59.0,  0.02067,      0,      0,       0,       0,       0,       0
total,        900297,   14910,   14910,   14910,     0.5,     0.3,     1.0,     3.5,    24.6,    95.5,   60.0,  0.02033,      0,      1,      84,      84,       0,     581
total,        917966,   17669,   17669,   17669,     0.4,     0.3,     1.0,     2.5,     7.9,    16.0,   61.0,  0.02003,      0,      0,       0,       0,       0,       0
total,        934485,   16519,   16519,   16519,     0.5,     0.3,     1.1,     3.1,     8.9,    27.4,   62.0,  0.01970,      0,      0,       0,       0,       0,       0
total,        934528,   10177,   10177,   10177,     0.6,     0.3,     1.6,     4.5,     4.5,     4.5,   62.0,  0.19411,      0,      0,       0,       0,       0,       0


Results:
Op rate                   :   15,072 op/s  [WRITE: 15,072 op/s]
Partition rate            :   15,072 pk/s  [WRITE: 15,072 pk/s]
Row rate                  :   15,072 row/s [WRITE: 15,072 row/s]
Latency mean              :    0.5 ms [WRITE: 0.5 ms]
Latency median            :    0.3 ms [WRITE: 0.3 ms]
Latency 95th percentile   :    1.2 ms [WRITE: 1.2 ms]
Latency 99th percentile   :    3.3 ms [WRITE: 3.3 ms]
Latency 99.9th percentile :   15.8 ms [WRITE: 15.8 ms]
Latency max               :  319.0 ms [WRITE: 319.0 ms]
Total partitions          :    934,528 [WRITE: 934,528]
Total errors              :          0 [WRITE: 0]
Total GC count            : 15
Total GC memory           : 8.578 GiB
Total GC time             :    1.6 seconds
Avg GC time               :  109.0 ms
StdDev GC time            :   42.1 ms
Total operation time      : 00:01:02

END
```


16 thread - shotover-proxy - with release optimisations
```
Running WRITE with 8 threads until stderr of mean < 0.02
type       total ops,    op/s,    pk/s,   row/s,    mean,     med,     .95,     .99,    .999,     max,   time,   stderr, errors,  gc: #,  max ms,  sum ms,  sdv ms,      mb
total,          2375,    2375,    2375,    2375,     2.1,     1.8,     3.7,     5.3,    14.8,    15.3,    1.0,  0.00000,      0,      0,       0,       0,       0,       0
total,          8261,    5886,    5886,    5886,     1.3,     1.2,     2.1,     2.9,     8.1,    11.7,    2.0,  0.29949,      0,      0,       0,       0,       0,       0
total,         14220,    5959,    5959,    5959,     1.3,     1.2,     2.1,     3.1,     9.9,    11.3,    3.0,  0.20308,      0,      0,       0,       0,       0,       0
total,         21285,    7065,    7065,    7065,     1.1,     1.1,     1.6,     2.2,     2.9,     5.0,    4.0,  0.16462,      0,      0,       0,       0,       0,       0
total,         32573,   11288,   11288,   11288,     0.7,     0.6,     1.2,     1.6,     2.8,     8.5,    5.0,  0.19574,      0,      0,       0,       0,       0,       0
total,         45017,   12444,   12444,   12444,     0.6,     0.6,     0.9,     1.3,     2.9,     4.2,    6.0,  0.18522,      0,      0,       0,       0,       0,       0
total,         56307,   11290,   11290,   11290,     0.7,     0.6,     0.9,     1.4,     5.7,    98.7,    7.0,  0.16543,      0,      1,      90,      90,       0,     604
total,         67200,   10893,   10893,   10893,     0.7,     0.6,     1.2,     2.2,     5.8,     7.5,    8.0,  0.14383,      0,      0,       0,       0,       0,       0
total,         80102,   12902,   12902,   12902,     0.6,     0.6,     0.9,     1.1,     2.4,     3.0,    9.0,  0.13096,      0,      0,       0,       0,       0,       0
total,         92460,   12358,   12358,   12358,     0.6,     0.6,     0.9,     1.3,     2.7,     3.5,   10.0,  0.11851,      0,      0,       0,       0,       0,       0
total,        104999,   12539,   12539,   12539,     0.6,     0.6,     0.9,     1.3,     2.7,     3.4,   11.0,  0.10829,      0,      0,       0,       0,       0,       0
total,        116470,   11471,   11471,   11471,     0.7,     0.6,     0.9,     1.2,     3.1,    89.3,   12.0,  0.09959,      0,      1,      88,      88,       0,     586
total,        128736,   12266,   12266,   12266,     0.6,     0.6,     0.9,     1.2,     1.9,     2.9,   13.0,  0.09190,      0,      0,       0,       0,       0,       0
total,        141408,   12672,   12672,   12672,     0.6,     0.6,     0.9,     1.1,     2.4,     3.5,   14.0,  0.08557,      0,      0,       0,       0,       0,       0
total,        154007,   12599,   12599,   12599,     0.6,     0.6,     0.9,     1.1,     2.2,     3.5,   15.0,  0.07997,      0,      0,       0,       0,       0,       0
total,        166847,   12840,   12840,   12840,     0.6,     0.6,     0.9,     1.1,     1.4,     2.1,   16.0,  0.07517,      0,      0,       0,       0,       0,       0
total,        178489,   11642,   11642,   11642,     0.7,     0.6,     0.9,     1.3,     2.7,    78.4,   17.0,  0.07078,      0,      1,      77,      77,       0,     601
total,        190574,   12085,   12085,   12085,     0.6,     0.6,     1.0,     1.3,     2.8,     4.5,   18.0,  0.06673,      0,      0,       0,       0,       0,       0
total,        202554,   11980,   11980,   11980,     0.7,     0.6,     1.0,     1.3,     2.5,     4.6,   19.0,  0.06311,      0,      0,       0,       0,       0,       0
total,        214543,   11989,   11989,   11989,     0.7,     0.6,     0.9,     1.3,     2.2,     3.1,   20.0,  0.05986,      0,      0,       0,       0,       0,       0
total,        227115,   12572,   12572,   12572,     0.6,     0.6,     0.9,     1.1,     1.6,     2.0,   21.0,  0.05703,      0,      0,       0,       0,       0,       0
total,        238615,   11500,   11500,   11500,     0.7,     0.6,     0.9,     1.2,     4.0,    84.5,   22.0,  0.05445,      0,      1,      82,      82,       0,     604
total,        248225,    9610,    9610,    9610,     0.8,     0.7,     1.3,     1.9,     3.5,     4.7,   23.0,  0.05266,      0,      0,       0,       0,       0,       0
total,        260223,   11998,   11998,   11998,     0.7,     0.6,     1.0,     1.2,     2.5,     3.2,   24.0,  0.05041,      0,      0,       0,       0,       0,       0
total,        271832,   11609,   11609,   11609,     0.7,     0.6,     1.0,     1.2,     1.5,     1.9,   25.0,  0.04834,      0,      0,       0,       0,       0,       0
total,        283807,   11975,   11975,   11975,     0.7,     0.6,     1.0,     1.3,     3.2,     4.5,   26.0,  0.04644,      0,      0,       0,       0,       0,       0
total,        294124,   10317,   10317,   10317,     0.7,     0.7,     1.2,     1.8,     2.6,     3.6,   27.0,  0.04491,      0,      1,      97,      97,       0,     585
total,        303453,    9329,    9329,    9329,     0.9,     0.7,     1.4,     2.1,     3.4,   101.6,   28.0,  0.04346,      0,      0,       0,       0,       0,       0
total,        315380,   11927,   11927,   11927,     0.7,     0.6,     1.0,     1.2,     1.8,     3.4,   29.0,  0.04193,      0,      0,       0,       0,       0,       0
total,        326512,   11132,   11132,   11132,     0.7,     0.7,     1.0,     1.3,     2.6,     3.4,   30.0,  0.04053,      0,      0,       0,       0,       0,       0
total,        337026,   10514,   10514,   10514,     0.7,     0.7,     1.1,     1.6,     2.9,     4.0,   31.0,  0.03932,      0,      0,       0,       0,       0,       0
total,        346562,    9536,    9536,    9536,     0.8,     0.7,     1.1,     1.5,     2.6,    95.8,   32.0,  0.03817,      0,      1,      94,      94,       0,     600
total,        358085,   11523,   11523,   11523,     0.7,     0.6,     1.0,     1.3,     2.2,     3.2,   33.0,  0.03699,      0,      0,       0,       0,       0,       0
total,        370002,   11917,   11917,   11917,     0.7,     0.6,     1.0,     1.3,     2.1,     4.8,   34.0,  0.03589,      0,      0,       0,       0,       0,       0
total,        381537,   11535,   11535,   11535,     0.7,     0.6,     1.0,     1.4,     4.1,    10.9,   35.0,  0.03485,      0,      0,       0,       0,       0,       0
total,        393679,   12142,   12142,   12142,     0.6,     0.6,     0.9,     1.2,     2.8,     3.6,   36.0,  0.03389,      0,      0,       0,       0,       0,       0
total,        404734,   11055,   11055,   11055,     0.7,     0.6,     0.9,     1.2,     2.1,    84.1,   37.0,  0.03297,      0,      1,      77,      77,       0,     591
total,        416464,   11730,   11730,   11730,     0.7,     0.6,     1.0,     1.3,     2.9,     3.5,   38.0,  0.03209,      0,      0,       0,       0,       0,       0
total,        428148,   11684,   11684,   11684,     0.7,     0.6,     1.0,     1.3,     2.5,     3.6,   39.0,  0.03125,      0,      0,       0,       0,       0,       0
total,        440073,   11925,   11925,   11925,     0.7,     0.6,     1.0,     1.2,     2.6,     3.3,   40.0,  0.03046,      0,      0,       0,       0,       0,       0
total,        452236,   12163,   12163,   12163,     0.6,     0.6,     0.9,     1.2,     1.6,     2.2,   41.0,  0.02973,      0,      0,       0,       0,       0,       0
total,        463328,   11092,   11092,   11092,     0.7,     0.6,     0.9,     1.3,     5.3,    82.6,   42.0,  0.02902,      0,      1,      81,      81,       0,     576
total,        474902,   11574,   11574,   11574,     0.7,     0.6,     1.0,     1.3,     2.4,     3.1,   43.0,  0.02834,      0,      0,       0,       0,       0,       0
total,        486783,   11881,   11881,   11881,     0.7,     0.6,     1.0,     1.2,     2.0,     3.6,   44.0,  0.02769,      0,      0,       0,       0,       0,       0
total,        496513,    9730,    9730,    9730,     0.8,     0.7,     1.2,     1.9,     4.9,     8.3,   45.0,  0.02731,      0,      0,       0,       0,       0,       0
total,        506524,   10011,   10011,   10011,     0.8,     0.7,     1.2,     1.6,     3.3,     4.2,   46.0,  0.02687,      0,      0,       0,       0,       0,       0
total,        515364,    8840,    8840,    8840,     0.9,     0.8,     1.5,     2.6,     8.1,    11.1,   47.0,  0.02677,      0,      0,       0,       0,       0,       0
total,        523018,    7654,    7654,    7654,     1.0,     0.8,     1.4,     2.0,     5.3,   135.7,   48.0,  0.02668,      0,      1,     128,     128,       0,     590
total,        532945,    9927,    9927,    9927,     0.8,     0.7,     1.3,     2.0,     3.7,     4.1,   49.0,  0.02627,      0,      0,       0,       0,       0,       0
total,        544547,   11602,   11602,   11602,     0.7,     0.6,     1.0,     1.3,     3.0,     5.5,   50.0,  0.02574,      0,      0,       0,       0,       0,       0
total,        556775,   12228,   12228,   12228,     0.6,     0.6,     0.9,     1.1,     1.5,     3.7,   51.0,  0.02527,      0,      0,       0,       0,       0,       0
total,        568857,   12082,   12082,   12082,     0.6,     0.6,     0.9,     1.2,     2.1,     3.4,   52.0,  0.02479,      0,      0,       0,       0,       0,       0
total,        579655,   10798,   10798,   10798,     0.7,     0.6,     1.0,     1.3,     2.3,     3.4,   53.0,  0.02435,      0,      1,      94,      94,       0,     599
total,        589961,   10306,   10306,   10306,     0.8,     0.7,     1.2,     1.9,     3.6,    96.5,   54.0,  0.02389,      0,      0,       0,       0,       0,       0
total,        602032,   12071,   12071,   12071,     0.6,     0.6,     0.9,     1.2,     1.5,     3.2,   55.0,  0.02347,      0,      0,       0,       0,       0,       0
total,        613990,   11958,   11958,   11958,     0.7,     0.6,     0.9,     1.3,     2.7,     3.6,   56.0,  0.02306,      0,      0,       0,       0,       0,       0
total,        625842,   11852,   11852,   11852,     0.7,     0.6,     1.0,     1.3,     2.5,     3.1,   57.0,  0.02265,      0,      0,       0,       0,       0,       0
total,        637447,   11605,   11605,   11605,     0.7,     0.6,     1.0,     1.3,     2.4,     3.4,   58.0,  0.02226,      0,      0,       0,       0,       0,       0
total,        647939,   10492,   10492,   10492,     0.7,     0.6,     1.0,     1.3,     2.9,   110.9,   59.0,  0.02188,      0,      1,     104,     104,       0,     571
total,        659893,   11954,   11954,   11954,     0.7,     0.6,     0.9,     1.2,     1.6,     2.9,   60.0,  0.02152,      0,      0,       0,       0,       0,       0
total,        672079,   12186,   12186,   12186,     0.6,     0.6,     0.9,     1.2,     1.9,     2.7,   61.0,  0.02118,      0,      0,       0,       0,       0,       0
total,        684231,   12152,   12152,   12152,     0.6,     0.6,     0.9,     1.2,     2.3,     2.8,   62.0,  0.02085,      0,      0,       0,       0,       0,       0
total,        695603,   11372,   11372,   11372,     0.7,     0.6,     1.0,     1.3,     2.8,     3.8,   63.0,  0.02052,      0,      0,       0,       0,       0,       0
total,        706673,   11070,   11070,   11070,     0.7,     0.6,     0.9,     1.2,     2.0,    95.6,   64.0,  0.02022,      0,      1,      93,      93,       0,     587
total,        718291,   11618,   11618,   11618,     0.7,     0.6,     1.0,     1.4,     2.9,     4.6,   65.0,  0.01990,      0,      0,       0,       0,       0,       0
total,        718317,    8870,    8870,    8870,     0.8,     0.7,     1.2,     1.2,     1.2,     1.2,   65.0,  0.02015,      0,      0,       0,       0,       0,       0


Results:
Op rate                   :   11,051 op/s  [WRITE: 11,051 op/s]
Partition rate            :   11,051 pk/s  [WRITE: 11,051 pk/s]
Row rate                  :   11,051 row/s [WRITE: 11,051 row/s]
Latency mean              :    0.7 ms [WRITE: 0.7 ms]
Latency median            :    0.6 ms [WRITE: 0.6 ms]
Latency 95th percentile   :    1.1 ms [WRITE: 1.1 ms]
Latency 99th percentile   :    1.7 ms [WRITE: 1.7 ms]
Latency 99.9th percentile :    3.3 ms [WRITE: 3.3 ms]
Latency max               :  135.7 ms [WRITE: 135.7 ms]
Total partitions          :    718,317 [WRITE: 718,317]
Total errors              :          0 [WRITE: 0]
Total GC count            : 12
Total GC memory           : 6.927 GiB
Total GC time             :    1.1 seconds
Avg GC time               :   92.1 ms
StdDev GC time            :   13.4 ms
Total operation time      : 00:01:05

END
```

Latency is roughly an additional .3ms on the same host (median)

Proxy
Latency mean              :    0.7 ms [WRITE: 0.7 ms]
**Latency median            :    0.6 ms [WRITE: 0.6 ms]**
Latency 95th percentile   :    1.1 ms [WRITE: 1.1 ms]
Latency 99th percentile   :    1.7 ms [WRITE: 1.7 ms]
Latency 99.9th percentile :    3.3 ms [WRITE: 3.3 ms]
Latency max               :  135.7 ms [WRITE: 135.7 ms]

Direct
Latency mean              :    0.5 ms [WRITE: 0.5 ms]
**Latency median            :    0.3 ms [WRITE: 0.3 ms]**
Latency 95th percentile   :    1.2 ms [WRITE: 1.2 ms]
Latency 99th percentile   :    3.3 ms [WRITE: 3.3 ms]
Latency 99.9th percentile :   15.8 ms [WRITE: 15.8 ms]
Latency max               :  319.0 ms [WRITE: 319.0 ms]

Memory usage is pretty low at around 10MB during the cassandra-stress runs